#!/bin/bash
#
# CVS Info:
# $Author: markus $ 
# $Date: 2002/10/29 20:07:54 $ 
# $Revision: 1.8 $ 
#
# Patch description generator by Lars Ellenberg <l.g.e@web.de>
# for use with e.g. amorsu, by the same author, or 
# fou4s by Markus Gaugusch, see http://fou4s.gaugusch.at/
# or probably even yast2 online update, if you can tell it to look
# at those patch descriptions in the first place.
#
# Written in bash, not perl, since Markus likes bash better :)
# I use "hashes" anyways ...
#
# gpd uses rsync to generate the recursiv directory listings, so this
# obviously only works with rsync enabled servers...
#

if [ ! -f $1 ] ; then
	echo "Listing file $1 not found!"
	exit 1
fi
LISTING=$1 # use an already retrieved listing and don't get it with rsync

# get fou4s common functions
for fou4s in `dirname $0`/fou4s ./fou4s /usr/sbin/fou4s ; do
	test -x $fou4s && source $fou4s --commonfuncs && break
done
for mypath in `dirname $0` . /usr/bin ; do
	test -x $mypath/get_info_from_ARCHIVES.pl && break
done

if [ -z "$GPDLIST" ] ; then
	echo "Please edit $CONFIGFILE and put something into GpdList!!"
	exit 1
fi

# you may want to include DESC_SERIAL and RSYNCSERVER into fou4s config
# if not defined, they are defined here.
# ARCHIVES=$ARCHIVESDIR/ARCHIVES # fixme put into dlpath/.../suseversion!/
for a in $ARCHIVES ./ARCHIVES `dirname $0`/ARCHIVES /var/cache/fou4s/ARCHIVES ; do
	test -f $a && ARCHIVES=$a && break
done
if [ ! -f $ARCHIVES ] ; then
	echo
	echo "HINT: Copy ARCHIVES.gz to ${ARCHIVES%%ARCHIVES}, and gunzip it. This will make gpd.sh write better readable patch descriptions." | fmt
	ARCHIVES=
fi
DESC_SERIAL=G
test $VERBOSE -ge 1 && echo "Will use -$DESC_SERIAL as description serial number"
# RSYNCSERVER=rsync://ftp.gwdg.de/SuSE/ftp.suse.com/suse/$ARCH # fixme put into fou4s.conf
dotless_SUSEVERSION=${SUSEVERSION//./}

cat << _EOF

          Warning!
			 
          Updates from this tree are NOT necessarily 
          UPDATED,  EVEN WHEN VULNERABLE TO SECURITY 
          HOLES!

_EOF

function print_patch_description ()
{
	cat <<-__EOP__
	##
	## Patch description of patch $name
	##

	Kind: generated

	Shortdescription.english: ${SUMMARY:-no summary available}

	Longdescription.english:
	This update is available from
	$full_path

	This patch description was generated automaticaly from recursive
	directory listings by amorsu/fou4s patch description generator.
	No further information wrt the purpose of this update can be given.

	Comments, requests for features, and bug reports, go to l.g.e(amorsu)@web.de

	${LONGDESC:-No further description available}
	Hsilgne.noitpircsedgnol:

	Size: $((size/1024))

	## file timestamp on ftp server: $time
	Buildtime: $unix_time

	MinYaST1Version:
	MinYaST2Version:

	Packages:
	##
	## -----> $name <-----
	##
	Filename: $name.rpm
	Label: $SUMMARY
	Series:
	InstPath: $full_path
	Size: 0815 $size
	Buildtime: $unix_time
	BuiltFrom:
	Flag:
	RpmGroup:${GROUP:+ $GROUP}
	Copyright:
	Version: $version
	StartCommand:
	Obsoletes:
	Requires:
	Provides:
	Segakcap:
	__EOP__
}

# you may want to integrate this into fou4s
# expects path in the environment, splits it up into
# path, name, ver, rel
function extract_version ()
{
	name=${path##*/}
	path=${path%/*}
	rel=${name##*-}
	[ "$rel" == "$name" ] && return 1 # no '-' found
	rel=${rel%%.*}
	name=${name%-*}
	ver=${name##*-}
	[ "$ver" == "$name" ] && return 1 # no second '-' found
	name=${name%-*}
}

# declare --struct _hash_AZ [ ( name version path size timestamp )* ]
# shell arrays are not very performant, so I use many of them
# "hashing" with first and last letter of rpm name
# we should really do this in perl, I think.

function register_version ()
{
	local seen_version
	local hash_part=_hash_${name:0:1}${name: -1}
	hash_part=${hash_part//[^A-Za-z0-9]/_}
	eval set -- \${$hash_part[@]}
	local i=1 n=$#
	# newer bash versions could say
	# for (( i=1; i <= n; i+=5 )) ; do
	while [ "$i" -le "$n" ] ; do
		[ "${!i}" == "$name" ] && break
		let i=i+5
	done
	if [ "$i" -le "$n" ] ; then
		# already in list
		eval seen_version=\${$hash_part[i+1]}
		# if I would say  
		# !vcmp $ver-$rel \> $seen_version ; then
		# ^not             ^greater (no <= available)
		# I would report same versions, too
		if vcmp $ver-$rel \< $seen_version ; then
			printf "%40s  %-16s\n" "$name already seen" "$seen_version"
return 1
		else
			printf "%40s  %-16s\n" "$name $ver-$rel overrides" "$seen_version"
		fi
	fi
	# I know I don't use index 0
	eval $hash_part[i]=$name
	eval $hash_part[i+1]=$ver-$rel
	eval $hash_part[i+2]=$full_path
	eval $hash_part[i+3]=$size
	eval $hash_part[i+4]=\"$date $time\"
}

# do not use [[ ]] for compatibility
function parse_listing ()
{
	local perm size date time full_path path name ver rel
	local version unix_time

	# first loop, find all available rpm versions,
	# only consider the most recent version per basename
	while read perm size date time path ; do
		[ "$perm" == "#" ] && PACKAGE_PATH=$size \
		                                  && continue
		# following three lines could have been outsourced, if
		# rsync was piped through grep '^-.*.rpm$'
		# I just like to store the full listing
		# regular files, not symlinks or the like
		[ "${perm#-}" == "$perm" ]        && continue
		# rpms
		[ "${path%.rpm}" == "$path" ]     && continue
		# no source rpms
		[ "${path%.src.rpm}" == "$path" ] || continue
		if [ ${PACKAGE_PATH#update/$SUSEVERSION} != $PACKAGE_PATH ] ; then
			# this is for the update trees
			# should come up with something similar as the
			# descriptions in the patches{,.cont} directories
			full_path=$path
		else
			# have to use path relative to default update directory
			full_path=../../$PACKAGE_PATH/$path
		fi
		# if we cannot guess the version, ignore this file
		extract_version || continue
		test $VERBOSE -ge 1 && printf "%-40s  %-16s %s\n" $name $ver-$rel $path
		register_version
	done

	# second loop, print out descriptions for all found versions
	local hash_part rpm_info LONGDESC SUMMARY GOUP 
	echo "Removing previously generated patch descriptions"
	server=${RSYNCSERVER#rsync://}
	server=${server%%/*}
	rm $DLPATH/$server/$SERVERPATH/patches/*-$DESC_SERIAL 2>/dev/null
	echo "Generating new patch descriptions to $DLPATH/$server/$SERVERPATH"
	# newer bashes could say: 
	# for hash_part in ${!_hash_*} ; do
	for hash_part in $(set | awk -F= '/^_hash_..=/ { print $1; }') ; do
		eval set -- \"\${$hash_part[@]}\"
	
		while [ $# -gt 0 ]; do
			name=$1
			version=$2
			full_path=$3
			size=$4
			time=$5
			# is --utc a correct asumption?
			# it is a rough guess, anyways...
			unix_time=`date --utc -d "$5" +%s`
			shift 5
			LONGDESC=''
			SUMMARY=''
			GROUP=''
			rpm_info=''
			if [ -e "$ARCHIVES" ] ; then
				echo calling get_info with $name
				LONGDESC=`$mypath/get_info_from_ARCHIVES.pl $name 2>/dev/null`
				SUMMARY="${LONGDESC%%|*}"
				LONGDESC="${LONGDESC#$SUMMARY|}"
			else
				rpm_info=`rpm -q --queryformat "%{SUMMARY}|%{GROUP}" $name 2>/dev/null`
				if [ "$rpm_info" ] ; then
					SUMMARY=${rpm_info%%|}
					GROUP=${rpm_info##|}
				fi
			fi
			# echo $name $version $full_path $size $unix_time $SUMMARY $GROUP
			test $VERBOSE -ge 1 && echo Generating $name-$DESC_SERIAL
			print_patch_description > $DLPATH/$server/$SERVERPATH/patches/$name-$DESC_SERIAL
		done
	done
}

if [ ! -d $DLPATH/$server/$SERVERPATH/patches ] ; then
	echo "No directory $DLPATH/$server/$SERVERPATH/patches"
	echo "Please run/configure fou4s to create it auomatically."
#fixme let fou4s do checking??
	exit 1
fi

if [ "$LISTING" ] ; then
	parse_listing < $LISTING
else
	TMPFILE=`mktemp $DLPATH/tmp.rsync.list.XXXXXX`
	if [ ! -f $TMPFILE  ] ; then
		echo "Error creating tempfile in $DLPATH!!"
		exit 2
	fi
	echo "# $ARCH $SUSEVERSION $RSYNCSERVER $DLPATH/$server/$SERVERPATH/patches" > $TMPFILE
	echo Connecting to rsync server $RSYNCSERVER/
	start_time=`date +%s`
	for PACKAGE_PATH in $GPDLIST ; do
		echo "# $PACKAGE_PATH" >> $TMPFILE
		echo "Getting descriptions from $PACKAGE_PATH/"
		rsync -r $RSYNCSERVER/$PACKAGE_PATH/ | grep -v kernel | grep -v " zq1\/" >> $TMPFILE
		ret=$?
		# seems like rsync returns 11 for server response "no such directory"
		# some of the above listed locations are not available for all
		# arch/releases, so ignore it
		[ $ret -eq 0 -o $ret -eq 11 ] || exit -1

	done
	end_time=`date +%s`
	echo Downloaded in $((end_time-start_time)) seconds
	start_time=`date +%s`
	parse_listing < $TMPFILE
	end_time=`date +%s`
	echo Generated in $((end_time-start_time)) seconds
	rm $TMPFILE # fixme don't use when verbose > 2
fi
